@@ -13,42 +13,42 @@
 #
 # COPYRIGHT
 #   Copyright (C) 2000-2009 Andy Wardley.  All Rights Reserved.
+#   Copyright (C) 2009 Andrew Ford.  All Rights Reserved.
 #
 #   This module is free software; you can redistribute it and/or
 #   modify it under the same terms as Perl itself.
 #
 # REVISION
-#   $Id: POM.pm 71 2009-03-27 16:24:19Z ford $
+#   $Id: POM.pm 91 2013-12-31 07:36:02Z ford $
 #
 #========================================================================
 
 package Pod::POM;
-
-require 5.004;
+$Pod::POM::VERSION = '2.01';
+require 5.006;
 
 use strict;
+use warnings;
+
 use Pod::POM::Constants qw( :all );
 use Pod::POM::Nodes;
 use Pod::POM::View::Pod;
 
-use vars qw( $VERSION $DEBUG $ERROR $ROOT $TEXTSEQ $DEFAULT_VIEW );
-use base qw( Exporter );
+use parent qw( Exporter );
 
-$VERSION = '0.25';
-$DEBUG   = 0 unless defined $DEBUG;
-$ROOT    = 'Pod::POM::Node::Pod';               # root node class
-$TEXTSEQ = 'Pod::POM::Node::Sequence';          # text sequence class
-$DEFAULT_VIEW = 'Pod::POM::View::Pod';          # default view class
+our $DEBUG   = 0 unless defined $DEBUG;
+our $ROOT    = 'Pod::POM::Node::Pod';               # root node class
+our $TEXTSEQ = 'Pod::POM::Node::Sequence';          # text sequence class
+our $DEFAULT_VIEW = 'Pod::POM::View::Pod';          # default view class
 
 
 #------------------------------------------------------------------------
 # allow 'meta' to be specified as a load option to activate =meta tags
 #------------------------------------------------------------------------
 
-use vars qw( @EXPORT_FAIL @EXPORT_OK $ALLOW_META );
-@EXPORT_OK   = qw( meta );
-@EXPORT_FAIL = qw( meta );
-$ALLOW_META  = 0;
+our @EXPORT_OK   = qw( meta );
+our @EXPORT_FAIL = qw( meta );
+our $ALLOW_META  = 0;
 
 sub export_fail {
     my $class = shift;
@@ -174,6 +174,20 @@ sub parse_text {
     $$line  = 1;
     $inpod  = 0;
 
+    my @encchunks = split /^(=encoding.*)/m, $text;
+    $text = shift @encchunks;
+    while (@encchunks) {
+        my($encline,$chunk) = splice @encchunks, 0, 2;
+        require Encode;
+        my($encoding) = $encline =~ /^=encoding\s+(\S+)/;
+        if ($encoding ne 'utf8' || !Encode::is_utf8($chunk)) {
+	        Encode::from_to($chunk, $encoding, "utf8");
+	    }
+        Encode::_utf8_on($chunk);
+        # $text .= "xxx$encline";
+        $text .= $chunk;
+    }
+
 # patch from JJ    
 #    while ($text =~ /(?:(.*?)(\n{2,}))|(.+$)/sg) {
     while ($text =~ /(?:(.*?)((?:\s*\n){2,}))|(.+$)/sg) {
@@ -534,7 +548,7 @@ Pod::POM - POD Object Model
 
     # create custom view
     package My::View;
-    use base qw( Pod::POM::View::HTML );
+    use parent qw( Pod::POM::View::HTML );
 
     sub view_head1 {
 	my ($self, $item) = @_;
@@ -968,7 +982,7 @@ as well as regular, formatted text blocks to contain code sequences,
 such like C<BE<lt>thisE<gt>> and C<IE<lt>thisE<gt>>.  These are used
 to indicate different markup styles, mark external references or index
 items, and so on.  What's more, they can be C<BE<lt>nested
-IE<lt>indefinatelyE<gt>E<gt>>.  Pod::POM takes care of all this by
+IE<lt>indefinitelyE<gt>E<gt>>.  Pod::POM takes care of all this by
 parsing such text, along with any embedded sequences, into Yet Another
 Tree, the root node of which is a Pod::POM::Node::Text object,
 possibly containing other Pod::POM::Node::Sequence objects.  When the
@@ -981,7 +995,7 @@ Now the real magic comes into effect.  You can define one view to
 render bold/italic text in one style:
 
     package My::View::Text;
-    use base qw( Pod::POM::View::Text );
+    use parent qw( Pod::POM::View::Text );
 
     sub view_seq_bold {
 	my ($self, $text) = @_;
@@ -996,7 +1010,7 @@ render bold/italic text in one style:
 And another view to render it in a different style:
 
     package My::View::HTML;
-    use base qw( Pod::POM::View::HTML );
+    use parent qw( Pod::POM::View::HTML );
 
     sub view_seq_bold {
 	my ($self, $text) = @_;
@@ -1391,7 +1405,7 @@ Content elements: over, begin, for, verbatim, text, code.
 
 The C<over> node encloses the Pod content in a list starting at an C<=over> 
 tag and continuing up to the matching C<=back> tag.  Lists may be nested 
-indefinately.
+indefinitely.
 
 Attributes: indent (default: 4)
 
@@ -1590,10 +1604,6 @@ Andy Wardley E<lt>abw@kfs.orgE<gt>
 
 Andrew Ford E<lt>A.Ford@ford-mason.co.ukE<gt> (co-maintainer as of 03/2009)
 
-=head1 VERSION
-
-This is version 0.25 of the Pod::POM module.
-
 =head1 COPYRIGHT
 
 Copyright (C) 2000-2009 Andy Wardley.  All Rights Reserved.
@@ -1618,3 +1628,4 @@ further details:
 
     http://www.cpan.org/modules/by-module/Pod/
 
+=cut
