@@ -6,29 +6,31 @@
 #   Base class for a node in a Pod::POM tree.
 #
 # AUTHOR
-#   Andy Wardley   <abw@kfs.org>
+#   Andy Wardley   <abw@wardley.org>
 #
 # COPYRIGHT
-#   Copyright (C) 2000, 2001 Andy Wardley.  All Rights Reserved.
+#   Copyright (C) 2000-2003 Andy Wardley.  All Rights Reserved.
 #
 #   This module is free software; you can redistribute it and/or
 #   modify it under the same terms as Perl itself.
 #
 # REVISION
-#   $Id: Node.pm,v 1.3 2002/02/06 16:45:23 abw Exp $
+#   $Id: Node.pm 91 2013-12-31 07:36:02Z ford $
 #
 #========================================================================
 
 package Pod::POM::Node;
-
-require 5.004;
+$Pod::POM::Node::VERSION = '2.01';
+require 5.006;
 
 use strict;
+use warnings;
+
 use Pod::POM::Nodes;
-use Pod::POM::Constants qw( :status );
-use vars qw( $VERSION $DEBUG $ERROR $NODES $NAMES $AUTOLOAD );
+use Pod::POM::Constants qw( :all );
+use vars qw( $DEBUG $ERROR $NODES $NAMES $AUTOLOAD );
+use constant DUMP_LINE_LENGTH => 80;
 
-$VERSION = sprintf("%d.%02d", q$Revision: 1.3 $ =~ /(\d+)\.(\d+)/);
 $DEBUG   = 0 unless defined $DEBUG;
 $NODES   = {
     pod      => 'Pod::POM::Node::Pod',
@@ -49,7 +51,10 @@ $NAMES = {
 };
 
 # overload stringification to present node via a view
-use overload '""' => 'present';
+use overload 
+    '""'     => 'present',
+    fallback => 1,
+    'bool'   => sub { 1 };
 
 # alias meta() to metadata()
 *meta = \*metadata;
@@ -60,22 +65,22 @@ use overload '""' => 'present';
 # 
 # Constructor method.  Returns a new Pod::POM::Node::* object or undef
 # on error.  First argument is the Pod::POM parser object, remaining 
-# arguments are node attributes as specified in %ATTRIBS in derived class
+# arguments are node attributes as specified in @ATTRIBS in derived class
 # package.
 #------------------------------------------------------------------------
 
 sub new {
     my $class = shift;
     my $pom   = shift;
-    my ($type, $attribs, $accept, $key, $value, $default);
+    my ($type, @attribs, $accept);
 
     $type = $NAMES->{ $class };
 
     {
 	no strict qw( refs );
-        $attribs = \%{"$class\::ATTRIBS"} || [ ];
+        @attribs = @{"$class\::ATTRIBS"};
 	$accept  = \@{"$class\::ACCEPT"}  || [ ];
-	unless (defined (%{"$class\::ACCEPT"})) {
+	unless (%{"$class\::ACCEPT"}) {
 	    %{"$class\::ACCEPT"} = ( 
 		map { ( $_ => $NODES->{ $_ } ) } @$accept,
 	    );
@@ -91,9 +96,8 @@ sub new {
     }, $class;
 
     # set attributes from arguments
-    keys %$attribs;	    # reset hash iterator
-    while(my ($key, $default) = each %$attribs) {
-	$value = shift || $default;
+    while(my ($key, $default) = splice(@attribs, 0, 2)) {
+	my $value = shift || $default;
 	return $class->error("$type expected a $key")
 	    unless $value;
 	$self->{ $key } = $value;
@@ -143,12 +147,14 @@ sub add {
 	    unless defined $node;
 	push(@{ $self->{ $type   } }, $node);
 	push(@{ $self->{ content } }, $node);
+    $pom->{in_begin} = 1 if $nodeclass eq 'Pod::POM::Node::Begin';
 	return $node;
     }
 
     # REDUCE: expect indicates the token that should terminate this node
     if (defined $expect && ($type eq $expect)) {
 	DEBUG("$name terminated by expected $type\n");
+    $pom->{in_begin} = 0 if $name eq 'begin';
 	return REDUCE;
     }
 
@@ -178,8 +184,13 @@ sub present {
     $view    ||= $Pod::POM::DEFAULT_VIEW;
     my $type   = $self->{ type };
     my $method = "view_$type";
-    DEBUG("presenting method $method\n");
-    return $view->$method($self, @args);
+    DEBUG("presenting method $method to $view\n");
+    my $txt = $view->$method($self, @args);
+    if ($view->can("encode")){
+        return $view->encode($txt);
+    } else {
+        return $txt;
+    }
 }
 
 
@@ -218,7 +229,7 @@ sub metadata {
 # package variable $ERROR (class method) or internal member 
 # $self->{ _ERROR } (object method).  The presence of parameters indicates
 # that the error value should be set.  Undef is then returned.  In the
-# abscence of parameters, the current error value is returned.
+# absence of parameters, the current error value is returned.
 #------------------------------------------------------------------------
 
 sub error {
@@ -249,6 +260,90 @@ sub error {
 }
 
 
+#------------------------------------------------------------------------
+# dump()
+#
+# Returns a representation of the element and all its children in a 
+# format useful only for debugging.  The structure of the document is 
+# shown by indentation (inspired by HTML::Element).
+#------------------------------------------------------------------------
+
+sub dump {
+    my($self, $depth) = @_;
+    my $output;
+    $depth = 0 unless defined $depth;
+    my $nodepkg = ref $self;
+    if ($self->isa('REF')) {
+        $self = $$self;
+        my $cmd = $self->[CMD];
+        my @content = @{ $self->[CONTENT] };
+        if ($cmd) {
+            $output .= ("  " x $depth) . $cmd . $self->[LPAREN] . "\n";
+        }
+        foreach my $item (@content) {
+            if (ref $item) {
+                $output .= $item->dump($depth+1);  # recurse
+            }
+            else {  # text node
+		$output .= _dump_text($item, $depth+1);
+	    }
+	}
+	if ($cmd) {
+            $output .= ("  " x $depth) . $self->[RPAREN] . "\n", ;
+        }
+    }
+    else {
+        no strict 'refs';
+        my @attrs = sort keys %{{ @{"${nodepkg}::ATTRIBS"} }};
+        $output .= ("  " x $depth) . $self->type . "\n";
+        foreach my $attr (@attrs) {
+            if (my $value = $self->{$attr}) {
+                $output .= ("  " x ($depth+1)) . "\@$attr\n";
+                
+                if (ref $value) {
+                    $output .= $value->dump($depth+1);
+                }
+                else {
+                    $output .= _dump_text($value, $depth+2);
+                }
+            }
+        }
+        foreach my $item (@{$self->{content}}) {
+            if (ref $item) {  # element
+                $output .= $item->dump($depth+1);  # recurse
+            } 
+            else {  # text node
+		$output .= _dump_text($item, $depth+1);
+	    }
+	}
+    }
+
+    return $output;
+}
+
+sub _dump_text {
+    my ($text, $depth) = @_;
+
+    my $output       = "";
+    my $padding      = "  " x $depth;
+    my $max_text_len = DUMP_LINE_LENGTH - length($depth) - 2;
+
+    foreach my $line (split(/\n/, $text)) {
+	$output .= $padding;
+	if (length($line) > $max_text_len or $line =~ m<[\x00-\x1F]>) {
+	    # it needs prettyin' up somehow or other
+	    my $x = (length($line) <= $max_text_len) ? $_ : (substr($line, 0, $max_text_len) . '...');
+	    $x =~ s<([\x00-\x1F])>
+		<'\\x'.(unpack("H2",$1))>eg;
+	    $output .= qq{"$x"\n};
+	} else {
+	    $output .= qq{"$line"\n};
+	}
+    }
+    return $output;
+}
+
+
 #------------------------------------------------------------------------
 # AUTOLOAD
 #------------------------------------------------------------------------
@@ -293,10 +388,10 @@ Pod::POM::Node - base class for a POM node
 =head1 SYNOPSIS
 
     package Pod::POM::Node::Over;
-    use base qw( Pod::POM::Node );
-    use vars qw( %ATTRIBS @ACCEPT $EXPECT $ERROR );
+    use parent qw( Pod::POM::Node );
+    use vars qw( @ATTRIBS @ACCEPT $EXPECT $ERROR );
 
-    %ATTRIBS =   ( indent => 4 );
+    @ATTRIBS =   ( indent => 4 );
     @ACCEPT  = qw( over item begin for text verbatim );
     $EXPECT  =  q( back );
 
@@ -318,7 +413,7 @@ This module implements a base class node which is subclassed to
 represent different elements within a Pod Object Model. 
 
     package Pod::POM::Node::Over;
-    use base qw( Pod::POM::Node );
+    use parent qw( Pod::POM::Node );
 
 The base class implements the new() constructor method to instantiate 
 new node objects.  
@@ -326,16 +421,16 @@ new node objects.
     my $list = Pod::POM::Node::Over->new();
 
 The characteristics of a node can be specified by defining certain
-variables in the derived class package.  The C<%ATTRIBS> hash can be
+variables in the derived class package.  The C<@ATTRIBS> list can be
 used to denote attributes that the node should accept.  In the case of
 an C<=over> node, for example, an C<indent> attribute can be specified
 which otherwise defaults to 4.
 
     package Pod::POM::Node::Over;
-    use base qw( Pod::POM::Node );
-    use vars qw( %ATTRIBS $ERROR );
+    use parent qw( Pod::POM::Node );
+    use vars qw( @ATTRIBS $ERROR );
 
-    %ATTRIBS = ( indent => 4 );
+    @ATTRIBS = ( indent => 4 );
 
 The new() method will now expect an argument to set the indent value, 
 or will use 4 as the default if no argument is provided.
@@ -346,10 +441,10 @@ or will use 4 as the default if no argument is provided.
 If the default value is undefined then the argument is mandatory.
 
     package Pod::POM::Node::Head1;
-    use base qw( Pod::POM::Node );
-    use vars qw( %ATTRIBS $ERROR );
+    use parent qw( Pod::POM::Node );
+    use vars qw( @ATTRIBS $ERROR );
 
-    %ATTRIBS = ( title => undef );
+    @ATTRIBS = ( title => undef );
 
     package main;
     my $head = Pod::POM::Node::Head1->new('My Title');
@@ -374,10 +469,10 @@ The C<@ACCEPT> package variable can be used to indicate the node types
 that are permitted as children of a node.
 
     package Pod::POM::Node::Head1;
-    use base qw( Pod::POM::Node );
-    use vars qw( %ATTRIBS @ACCEPT $ERROR );
+    use parent qw( Pod::POM::Node );
+    use vars qw( @ATTRIBS @ACCEPT $ERROR );
 
-    %ATTRIBS =   ( title => undef );
+    @ATTRIBS =   ( title => undef );
     @ACCEPT  = qw( head2 over begin for text verbatim );
 
 The add() method can then be called against a node to add a new child
@@ -422,10 +517,10 @@ such a match is made, the add() method returns REDUCE to indicate
 successful termination.
 
     package Pod::POM::Node::Over;
-    use base qw( Pod::POM::Node );
-    use vars qw( %ATTRIBS @ACCEPT $EXPECT $ERROR );
+    use parent qw( Pod::POM::Node );
+    use vars qw( @ATTRIBS @ACCEPT $EXPECT $ERROR );
 
-    %ATTRIBS =   ( indent => 4 );
+    @ATTRIBS =   ( indent => 4 );
     @ACCEPT  = qw( over item begin for text verbatim );
     $EXPECT  =  q( back );
 
